/************************************************************

  Name:         CMIPObj.java

  Description:  CMIP implementation

  Coded by:     Luca Deri (lde@zurich.ibm.com)

  Licensed Materials - Property of IBM

  (C) Copyright IBM Corporation 1995-96.  All Rights Reserved.

**************************************************************/


import java.lang.*;
import java.util.*;
import java.net.*;
import java.io.*;

public class CMIPObj extends Information implements java.lang.Cloneable {
	private String agentAET, efdObjInst, rootDN;
	
	static private String CMIP_GET     = "CMIP_Get";
	static private String CMIP_SET     = "CMIP_Set";
	static private String CMIP_DELETE  = "CMIP_Delete";
	static private String CMIP_CREATE  = "CMIP_Create";
	static private String CMIP_ACTION  = "CMIP_Action";
	static private String CMIP_Evr     = "CMIP_Evr";
	static private String ASN_METADATA = "ASN_Metadata";
	static private String OID_MAPPING  = "OID_Mapping";
	static private String GET_ACTIONS  = "Get_Actions";
	static private String GET_NM_BIND  = "Get_NameBindings";

	public CMIPObj(String aet) { 
	  agentAET = new String(aet); 
	  efdObjInst = null;
          rootDN = null;
	}
	
	public void SetAgentAET(String aet) { 
	  agentAET = new String(aet); }

	public void SetObjectClass(String val) { 
	  SetAttribute("objectClass", val); }

	public String GetObjectClass() { 
	  return((String)GetAttribute("objectClass")); }
	
	public void SetObjectInstance(String val) { 
	  SetAttribute("objectInstance", val); }

	public String GetObjectInstance() { 
	  return((String)GetAttribute("objectInstance")); }
	
	/* ********************************** */	
	public void CMIPCreateObject() throws ProxyException { 
	  CMIPPerformSingleOperation(CMIP_CREATE, 0); }

	public void CMIPDeleteObject() throws ProxyException { 
	  CMIPPerformSingleOperation(CMIP_DELETE, 0); }

	public Vector CMIPDeleteContainedInstances() throws ProxyException { 
	  return(CMIPPerformScopedOperation(CMIP_DELETE, 0));}

	public void CMIPGetAttributes() throws ProxyException { 
	  CMIPPerformSingleOperation(CMIP_GET, 0); }

	public Vector CMIPGetContainedInstances() throws ProxyException { 
	  return(CMIPPerformScopedOperation(CMIP_GET, 0));}

	public void CMIPSetAttributes() throws ProxyException { 
	  CMIPPerformSingleOperation(CMIP_SET, 0);}

	public Vector CMIPSetContainedInstances() throws ProxyException { 
	  return(CMIPPerformScopedOperation(CMIP_SET, 0)); }

	public void CMIPPerformAction(int confirmed) throws ProxyException { 
	  CMIPPerformSingleOperation(CMIP_ACTION, confirmed);}

	public Vector CMIPPerformActionContainedInstances(int confirmed) throws ProxyException { 
	  return(CMIPPerformScopedOperation(CMIP_ACTION, confirmed)); }


	public int NotificationsAvailable() throws ProxyException {
	  String rs;
	  
	  if(rootDN == null)
	    throw(new ProxyException("Error: EFD not yet created"));
	  
	  rs = SendOffLineRequest(CMIP_Evr, "?command=notificationsAvailable"
				  +"&rootDN="+URLEncoder.encode(rootDN));
	  return((Integer.valueOf(rs)).intValue());		 	
	}
	
	public Information WaitForNotifications(int timeout) throws ProxyException { 
	  String s;

	  if(proxy == null)
	    throw(new ProxyException("Please specify what Proxy you want to use (UseProxy method)"));
	  
	  if(rootDN == null)
	    throw(new ProxyException("Error: EFD not yet created"));
	  
	  s = "?command=waitForNotifications&timeout="+timeout
	    +"&rootDN="+URLEncoder.encode(rootDN);
	  
	  // System.out.println("Called WaitForNotifications");
	  return(proxy.SendOffLineReqWithMultipleOut(asyncMode, CMIP_Evr, agentAET, s));
	}

	public void DeleteEFD() throws ProxyException {
	  String st, command = new String("?command=deleteEFD");
	  int idx;
	  
	  if(efdObjInst != null)
	    {
	      command = command.concat("&supInst="+URLEncoder.encode(efdObjInst));	      
	      command = command.concat("&rootDN="+URLEncoder.encode(rootDN));
	      
	      SendOffLineRequest(CMIP_Evr, command);
	      
	      efdObjInst = null;
	      rootDN = null;
	    }
	}

	public void CreateEFD(String sysInst, String filter) throws ProxyException {
	  String st, command = new String("?command=createEFD");
	  int idx;

	  if(efdObjInst != null)
	    throw(new ProxyException("Error: EFD created already"));

	  if(sysInst != null)
	    command = command.concat("&supInst="+URLEncoder.encode(sysInst));
	  
 	  if(filter != null)
	    command = command.concat("&filter="+URLEncoder.encode(filter));	 

	  st = SendOffLineRequest(CMIP_Evr, command);

	  if(st == null)
	    throw(new ProxyException("Error: unable to create EFD"));

	  idx = st.indexOf('#');

	  efdObjInst = st.substring(0, idx-1);
	  rootDN = st.substring(idx+1, st.length());
	}

	/* ********************************** */	

	public Information GetActions() throws ProxyException { 
	  if(proxy == null)
	    throw(new ProxyException("Please specify what Proxy you want to use (UseProxy method)"));
	  else	  
	  return(proxy.SendOffLineReqWithMultipleOut(asyncMode, GET_ACTIONS, 
						     agentAET, GetObjectClass())); 
	}

	public Information GetNameBindings() throws ProxyException {
	  if(proxy == null)
	    throw(new ProxyException("Please specify what Proxy you want to use (UseProxy method)"));
	  else
	  return(proxy.SendOffLineReqWithMultipleOut(asyncMode, GET_NM_BIND,
						     agentAET, GetObjectClass())); 
	}

	/* ********************************** */	

	public String GetSyntaxInfo(String syntax) throws ProxyException { 
	  return(SendOffLineRequest(ASN_METADATA, syntax)); }

	public String ConvertOID(String oid) throws ProxyException { 
	  return(SendOffLineRequest(OID_MAPPING, oid)); }

	/* ********************************** */	

	private String SendOffLineRequest(String type, String val) throws ProxyException { 

	  if(proxy == null)
	    throw(new ProxyException("Please specify what Proxy you want to use (UseProxy method)"));
	  else
	    return(proxy.SendOffLineRequest(asyncMode, type, agentAET, val)); 
	};

	private void CMIPPerformSingleOperation(String opName, int confirmed) throws ProxyException { 
	  Vector v;
	  Information rsp;
	
	  if(proxy == null)
	    throw(new ProxyException("Please specify what Proxy you want to use (UseProxy method)"));

	  RemoveAttribute("scope"); // -> Scope=default
	  RemoveAttribute("currentTime");
	  
	  if(confirmed == 1) SetAttribute("confirmedRequest", "1"); // Confirmed Operation
	    
	  v = proxy.SendRequest(asyncMode, opName, agentAET, this); // Context

	  if(confirmed == 1) RemoveAttribute("confirmedRequest");

	  // This is a single response
 	  if((v == null) || (v.size() != 1))
	    throw(new ProxyException("Liaison error: unexpected response ["+v+"]"));
	  
	  CopyValue((Information)v.firstElement());		
	};
	
	private Vector CMIPPerformScopedOperation(String opName, int confirmed) throws ProxyException { 
	  Vector v;
	
	  if(proxy == null)
	    throw(new ProxyException("Please specify what Proxy you want to use (UseProxy method)"));
	  
	  if(GetAttribute("scope") == null)
	    SetAttribute("scope", "(individualLevels 1)"); // Default

	  RemoveAttribute("currentTime");
	  if(confirmed == 1) SetAttribute("confirmedRequest", "1"); // Confirmed Operation
	    
	  v = proxy.SendRequest(asyncMode, opName, agentAET, this); // Context
	  
	  if(confirmed == 1) RemoveAttribute("confirmedRequest");

	  if(v == null) throw(new ProxyException("Communication problem: received empty rsp"));
	  
	  return(v);		
	};
	
    public synchronized String toString() {
	StringBuffer buf = new StringBuffer();
	buf.append("[agentAET="+agentAET+",");
	buf.append("attributes="+attributes.toString()+"]");
	return buf.toString();
   }
  
}
