/************************************************************

  Name:         SNMPObj.java

  Description:  SNMPObj class implementation

  Coded by:     Luca Deri (lde@zurich.ibm.com)

  Licensed Materials - Property of IBM

  (C) Copyright IBM Corporation 1995-96.  All Rights Reserved.

**************************************************************/

import java.lang.*;
import java.util.*;
import java.net.*;
import java.io.*;

public class SNMPObj extends Information implements java.lang.Cloneable {
	private String snmpAgtAddr;
	private int snmpAgtPort, trapSockId;
	
	static private String SNMP_GET         = "SNMP_Get";
	static private String SNMP_GET_NEXT    = "SNMP_GetNext";
	static private String SNMP_SET         = "SNMP_Set";
	static private String SNMP_WALK        = "SNMP_Walk";
	static private String SNMP_ATTR_INFO   = "SNMP_Attr_Info";
	static private String SNMP_OID_MAPPING = "SNMP_OID_Map";
	static private String SNMP_TRAP        = "SNMP_Trap";

	public SNMPObj() { 
	  snmpAgtAddr= new String("0.0.0.0");
	  snmpAgtPort = 160;
	  trapSockId = -1;
	}
	
	public SNMPObj(String remAgt, int remAgtPort) { 
	  snmpAgtAddr = new String(remAgt);
	  snmpAgtPort = remAgtPort;
	  trapSockId = -1;
	}
	
	/* ********************************** */	
	public void SetCommunity(String val) { 
	  SetAttribute("community", val); 
	}

	public void SetSnmpAgentAddress(String remAgt, int remAgtPort) {
	  snmpAgtAddr= new String(remAgt);
	  snmpAgtPort =  remAgtPort;
	}

	public Vector SNMPWalk() throws UnknownHostException, IOException, ProxyException { 
	  Vector v;
	
	  if(proxy == null)
	    throw(new ProxyException("Please specify what Proxy you want to use (UseProxy method)"));

	  if(snmpAgtPort != 160) /* standard port */
	    SetAttribute("agentPort", String.valueOf(snmpAgtPort)); // Default

	  v = proxy.SendRequest(asyncMode, SNMP_WALK, snmpAgtAddr, this); // Context
	  if(v == null) throw(new ProxyException("Internal error: received empty rsp"));	  
	  return(v);		
	}
	
	public void SNMPGetAttribute() throws UnknownHostException, IOException, ProxyException { 
	  RemoveAttribute("type");SNMPPerformSingleOperation(SNMP_GET); 
	  }
	
	public void SNMPGetNextAttribute() throws UnknownHostException, IOException, ProxyException { 
	  RemoveAttribute("type");SNMPPerformSingleOperation(SNMP_GET_NEXT); 
	}
	
	public void SNMPSetAttribute() throws UnknownHostException, IOException, ProxyException { 
	  SNMPPerformSingleOperation(SNMP_SET); 
	}

	public String SNMPGetAttributeInfo(String syntax) throws
	  UnknownHostException, IOException, ProxyException { 
	    return(SendOffLineRequest(SNMP_ATTR_INFO, syntax)); 
	  }

	public String ConvertOID(String oid) throws UnknownHostException, IOException, ProxyException { 
	    return(SendOffLineRequest(SNMP_OID_MAPPING, oid)); 
	  }

	/* ********************************** */	

	private String SendOffLineRequest(String type, String val) throws UnknownHostException, 
	IOException, ProxyException { 

	  if(proxy == null)
	    throw(new ProxyException("Please specify what Proxy you want to use (UseProxy method)"));
	  else
	    return(proxy.SendOffLineRequest(asyncMode, type, null, val)); 
	};

	public int TrapsAvailable() throws UnknownHostException, IOException, ProxyException {
	  String rs;
	  
	  if(trapSockId == -1)
	    throw(new ProxyException("trapListener not yet created"));

	  rs = SendOffLineRequest(SNMP_TRAP, "?command=trapsAvailable&socketId="+trapSockId);
	  return((Integer.valueOf(rs)).intValue());
	}

	public void CreateTrapListener(int port) throws UnknownHostException, IOException, ProxyException {
	  String rs;
	  
	  if(trapSockId != -1)
	    throw(new ProxyException("trapListener created already"));
	  
	  rs = SendOffLineRequest(SNMP_TRAP, "?command=createTrapListener&trapPort="+port);
	  trapSockId = (Integer.valueOf(rs)).intValue();
	}


	public void DeleteTrapListener() throws UnknownHostException, IOException, ProxyException {
	  String rs;
	  
	  if(trapSockId == -1)
	    throw(new ProxyException("trapListener not yet created"));
		  
	  rs = SendOffLineRequest(SNMP_TRAP, "?command=deleteTrapListener&socketId="+trapSockId);
	  trapSockId = -1;
	}

	public Information WaitForTraps(int timeout) throws UnknownHostException, IOException, ProxyException {
	  String s;
	  
	  if(proxy == null)
	    throw(new ProxyException("Please specify what Proxy you want to use (UseProxy method)"));
	  
	  if(trapSockId == -1)
	    throw(new ProxyException("trapListener not yet created"));
	  
	  s = "?command=waitForTraps&timeout="+timeout+"&socketId="+trapSockId;
	  
	  // System.out.println("Called WaitForTraps");
	  return(proxy.SendOffLineReqWithMultipleOut(asyncMode, SNMP_TRAP, "", s));
	}


	private void SNMPPerformSingleOperation(String opName) throws ProxyException { 
	  Vector v;
	  Information rsp;
	
	  if(proxy == null)
	    throw(new ProxyException("Please specify what Proxy you want to use (UseProxy method)"));

	  if(snmpAgtPort != 160) /* standard port */
	    SetAttribute("agentPort", String.valueOf(snmpAgtPort)); // Default

	  v = proxy.SendRequest(asyncMode, opName, snmpAgtAddr, this); // Context

	  //System.out.println(v);

	  // This is a single response
	  if((v == null) || (v.size() != 1))
	    throw(new ProxyException("Internal error: supposed to receive a single response ["+v+"]"));
	  
	  CopyValue((Information)v.firstElement());		
	};
		
    public synchronized String toString() {
	StringBuffer buf = new StringBuffer();
	buf.append("[snmpAgtAddr="+snmpAgtAddr+","+"snmpAgtPort="+snmpAgtPort +",");
	buf.append("proxy="+proxy+",attributes="+attributes.toString()+"]");
	return buf.toString();
   }
  
}
